const initialState = {
    statementConfig: [],
    loadingStatementConfig: false,

    actionTransfer: false,
    actionInputCashbox: false,
    actionOutputCashbox: false,
    actionStatementConfirm: false
}

const state = {
    statementConfig: initialState.statementConfig,
    loadingStatementConfig: initialState.loadingStatementConfig,

    actionTransfer: initialState.actionTransfer,
    actionInputCashbox: initialState.actionInputCashbox,
    actionOutputCashbox: initialState.actionOutputCashbox,
    actionStatementConfirm: initialState.actionStatementConfirm
}

const getters = {
    statementConfig(state) {
        return state.statementConfig
    },
    loadingStatementConfig(state) {
        return state.loadingStatementConfig
    },

    actionTransfer(state) {
        return state.actionTransfer
    },
    actionInputCashbox(state) {
        return state.actionInputCashbox
    },
    actionOutputCashbox(state) {
        return state.actionOutputCashbox
    },
    actionStatementConfirm(state) {
        return state.actionStatementConfirm
    }
}

const mutations = {
    setStatementConfig(state, payload) {
        state.statementConfig = payload
    },
    setLoadingStatementConfig(state, payload) {
        state.loadingStatementConfig = !!payload
    },

    /*  @ Transfer */
    setActionTransfer(state, payload) {
        let config = state.statementConfig.length > 0 ?  state.statementConfig[0] :false
        if (!config){
            state.actionTransfer = false
            return
        }
        let result =  !!config.transfer.userIds.find(item => item === payload.userSession) && !!config.transfer.statementIds.find(item => item === payload.typeStatement.id)
        state.actionTransfer = result
    },

    /* @ Input */
    setActionInputCashbox(state, payload) {
        let config = state.statementConfig.length > 0 ?  state.statementConfig[0] :false
        if (!config){
            state.actionInputCashbox = false
            return
        }
        let result =  !!config.inputCashBox.userIds.find(item => item === payload.userSession) && !!config.inputCashBox.statementIds.find(item => item === payload.typeStatement.id)
        state.actionInputCashbox = result
    },

    /*  @Output */
    setActionOutputCashbox(state, payload) {
        let config = state.statementConfig.length > 0 ?  state.statementConfig[0] :false
        if (!config) {
            state.actionOutputCashbox = false
            return
        }
        let result =  !!config.outputCashBox.userIds.find(item => item === payload.userSession) && !!config.outputCashBox.statementIds.find(item => item === payload.typeStatement.id)
        state.actionOutputCashbox = result
    },
    /* @Confirm */
    setActionStatementConfirm(state, payload) {
        let config = state.statementConfig.length > 0 ?  state.statementConfig[0] :false
        if (!config){
            state.actionStatementConfirm = false
            return
        }
        let result =  !!config.statementConfirm.userIds.find(item => item === payload.userSession)
        state.actionStatementConfirm = result
    }
}

/*
'transfer':{
    'userIds'
    'statementIds'
    'negativeAmount':
},
'inputCashBox': {
    'userIds',
    'statementIds'
},
'outputCashBox': {
    'userIds'
    'statementIds'
    'negativeAmount'
},

'delete': {
    'outputUserIds'
    'inputUserIds'
    'transferUserIds'
},
'statementConfirm' :{
    'userIds'
    'transferUserIds'
    'balanceUserIds'
    'negativeAmountUserIds'
},
'statementOpen',
'statementCancelUserIds',
'statementUnlinkUserIds'
 */


const actions = {
    initStatementConfig({ commit }, payload) {
        commit('setStatementConfig',payload)
        commit('setLoadingStatementConfig', payload)
    },
    verifyActionCashbox({ getters, commit }, payload){
        // if (getters.stateStatementSelected === 'open')
        commit('setActionTransfer', payload) // transfer
        commit('setActionInputCashbox', payload) // Input
        commit('setActionOutputCashbox', payload) // Output
        commit('setActionStatementConfirm', payload) //Confirm
    },
}

export default {
    state,
    getters,
    mutations,
    actions
}
